//******************************************************************************
//
//	LF functions
//
//******************************************************************************

#include  <msp430x12x2.h>
#include  <defines.h>

// Send SPI Transponder Access Command page 4: Serial nbr, Signature, Read addr
void TrpAccess_p4(void)
{
	ucSPITxData[0] = SPITAC;
	ucSPITxData[1] = 0x10;									// Write Adress: Authentication page 4
	ucSPITxData[2] = ucLFRxData[4];
	ucSPITxData[3] = ucLFRxData[3];
	ucSPITxData[4] = ucLFRxData[2];
	ucSPITxData[5] = ucLFRxData[1];
	ucSPITxData[6] = ucLFRxData[0];
	
	transmit_SPI(0x07, ucSPITxData);
	Wait100us(1);											// Wait for 100us, else first byte is not captured	
	receive_SPI(0x07);
	
	// Store Data to LFTx Buffer: LSByte received first via SPI
	ucLFTxData[1] = ucSPIRxData[0];							//	Serial nbr	LSByte	
	ucLFTxData[2] = ucSPIRxData[1];							//	Serial nbr
	ucLFTxData[3] = ucSPIRxData[2];							//	Serial nbr  MSByte	
	ucLFTxData[4] = ucSPIRxData[3];							//	Signature	LSByte	
	ucLFTxData[5] = ucSPIRxData[4];							//	Signature
	ucLFTxData[6] = ucSPIRxData[5];							//	Signature	MSByte	
	ucLFTxData[7] = ucSPIRxData[6];							//	Read addr
}

// Send SPI CRC Command: calculate CRC for Serial nbr, Signature, Read addr
void CRC_calculation(void)
{
	ucLFTxData[0]  = 0x7E;									// Add LF Start byte
	
	ucSPITxData[0] = SPICRC;
	ucSPITxData[1] = 0x07;									//  # of bytes
	ucSPITxData[2] = ucLFTxData[1];							//	Serial nbr	LSByte	
	ucSPITxData[3] = ucLFTxData[2];							//	Serial nbr
	ucSPITxData[4] = ucLFTxData[3];							//	Serial nbr  MSByte
	ucSPITxData[5] = ucLFTxData[4];							//	Signature	LSByte	
	ucSPITxData[6] = ucLFTxData[5];							//	Signature
	ucSPITxData[7] = ucLFTxData[6];							//	Signature	MSByte	
	ucSPITxData[8] = ucLFTxData[7];							//	Read addr
	
	transmit_SPI(0x09, ucSPITxData);
	Wait100us(1);											// Wait for 100us, else first byte is not captured	
	receive_SPI(0x02);
	
	//LSByte received first via SPI
	ucLFTxData[8] = ucSPIRxData[0];							// LSByte
	ucLFTxData[9] = ucSPIRxData[1];							// MSByte
}

unsigned char RSSI_measurement()
{
	unsigned int uiRSSI[3];
	unsigned char Driver;
	
	// Enable non-selected WDEs
	ucSPITxData[0] = SPIWDEs_ON;
	transmit_SPI(0x01, ucSPITxData);
	
	// Initialize ADC for RSSI measurement 
	ADC10CTL0 = SREF_1 + ADC10SHT_3 + ADC10ON + ADC10IE + REF2_5V + REFON; 
    ADC10CTL1 = INCH_5 + ADC10DIV_1 + ADC10SSEL_0;    		// P3.0 ADC option select with ADC10 Oscillator
    ADC10AE   = ADC10AE5;   						  		// Analog input enable
    Waitms(2);												// Wait 2 ms untill all WDEs are on (min 1 ms)
	
	// RSSI measurement RF1
	ucSPITxData[0] = SPIRSSI_1;
	transmit_SPI(0x01, ucSPITxData);
	P3OUT |= 0x80;											// P3.7 HIGH for RSSI measurement
	ADC10CTL0 |= ENC + ADC10SC;                       		// Sampling and conversion start
	_BIS_SR(LPM3_bits);										// Enter LPM3 and wait: if ADC finished, an interrupt is captured and LPM3 is quit
	uiRSSI[0] = 0x03FF - ADC10MEM;							// Read RSSI value[0x0000 - 0x03FF] from ADC register
	P3OUT &= ~0x80; 										// P3.7 LOW when RSSI measurement finished
	
	// RSSI measurement RF2
	ucSPITxData[0] = SPIRSSI_2;
	transmit_SPI(0x01, ucSPITxData);
	P3OUT |= 0x80;											// P3.7 HIGH
	ADC10CTL0 |= ENC + ADC10SC;                       		// Sampling and conversion start
	_BIS_SR(LPM3_bits);
	uiRSSI[1] = 0x03FF - ADC10MEM;						
	P3OUT &= ~0x80;											// P3.7 LOW
	
	// RSSI measurement RF3
	ucSPITxData[0] = SPIRSSI_3;
	transmit_SPI(0x01, ucSPITxData);
	P3OUT |= 0x80;											// P3.7 HIGH
	ADC10CTL0 |= ENC + ADC10SC;                       		// Sampling and conversion start
	_BIS_SR(LPM3_bits);
	uiRSSI[2] = 0x03FF - ADC10MEM;
	P3OUT &= ~0x80;											// P3.7 LOW
	
	// Disable non-selected WDEs, so that end of Powerburst 2 is detected on the right channel
	// and generates the final EOBA pulse
	ucSPITxData[0] = SPIWDEs_OFF;
	transmit_SPI(0x01, ucSPITxData);
	
	// Disabling ADC: First disable, then turn ADC off, else additional 160uA current consumption
	ADC10CTL0 &= ~ENC;								  		// Disable: ENC = 0
	ADC10CTL0 = 0x0000;								  		// Turn off: REFON = 0
	ADC10CTL1 = 0x0000;
    ADC10AE   = 0x0000;
	
	// Select Driver with best RSSI Result
	if(uiRSSI[0] > uiRSSI[1] && uiRSSI[0] > uiRSSI[2]) Driver = Driver1;
	else if(uiRSSI[1] > uiRSSI[0] && uiRSSI[1] > uiRSSI[2]) Driver = Driver2;
	else Driver = Driver3;
	
	return Driver;
}

void PEPS()
{
	int i, j;
	char next_bit, last_bit, shift;
	unsigned char Driver;
	
	P2IE = P2_EOBA;											// Enable interrupt for EOBA
	P2IES &= ~P2_EOBA;		 								// interrupt on rising edge
	interrupt_vector.field.eoba = 0;
	
	g_count = 0;
	CCR0 = 4000;											// Start Timer with timeout after 4000/4MHz = 1ms
	
	// Wait for first EOBA pulse
	while(interrupt_vector.field.eoba == 0 && g_count < 10);// g_count < 10 = Timeout after 10 ms
	
	// If EOBA pulse was detected
	if(interrupt_vector.field.eoba == 1 )
	{
		// Capture 5 bytes (40 bit challenge)
		for(j=0; j<5; j++)
		{
			//Detect byte
			CCR0 = 0;
			shift = 0x01;
			ucLFRxData[j] = 0x00;
			
			for(i=0; i<8; i++)
			{
				// Detect bit
				CCR0 = 2200;								// Set timer: timeout after 2200/4MHz = 550us
				g_count = 0;
				interrupt_vector.field.eoba = 0;
				while(interrupt_vector.field.eoba == 0 && g_count < 5);	// Wait for next EOBA pulse or timeout after 2.75 ms
				CCR0 = 0;	
				if(g_count == 1)							// If timeout: High bit (tbit > 550us) 
				{
					ucLFRxData[j] |= shift;
				}
				shift<<=1;
			}
		}
	
		// MSP430 access to Transponder: Takes about 14ms (Set Powerburst > 14ms)
	
		// Enable SPI
		P3SEL |= 0x0E;                            				// P3.1-3 SPI option selected
  		ME2 |= USPIE0;                            				// Enable USART0 SPI mode
  
		// Get Serial Number, Signature and Read Adress p4
  		TrpAccess_p4();
  		// Get CRC
 		CRC_calculation();
 		// Determine output driver
		Driver = RSSI_measurement();
		
		// Disable SPI Ports else ~30uA additional current consumption
		P3SEL &= ~0x0E;                            				// P3.1-3 SPI option not selected
  		ME2 &= ~USPIE0;                            				// Enable USART0 SPI mode
  		
  		// Detect last EOBA: End of Powerburst 2
		CCR0 = 12000;											// Set timer to 3ms
		g_count = 0;
		interrupt_vector.field.eoba = 0;
		
		while(interrupt_vector.field.eoba == 0 && g_count < 5);	// Timeout if no EOBA within 15 ms
		if(g_count < 5)
		{
			WDEEN_OFF;												// WDEEN LOW during LF transmission
		
			//Select Driver for transmission (RSSI result)
			switch(Driver)
			{
				case Driver1:	P2OUT |= P2_EN1;	break;			// Driver 1 HIGH
				case Driver2:	P2OUT |= P2_EN2;	break;			// Driver 2 HIGH
				case Driver3:	P1OUT |= P1_EN3;	break;			// Driver 3 HIGH
				}
	
			// Start LFTx transmission directly after last EOBA pulse
	
 			CCR2 = 15;
 			P3OUT &= ~P3_nLH;										// P3.6 LOW		
 			CCR0 = 29;
 			g_count = 0;
 			P1SEL |= P1_FTX;										// PWM on 
 	
  			// Send 16 LOW prebits
  			last_bit = 0;
 			while(g_count < 253);		 							// (256-3)/16 = 16 LOW prebits
 			g_count = 0;
 	
			// Send 10 Bytes
			for(j=0; j<10; j++)
			{
				shift = 0x01;
		
				// Send byte
				if(ucLFTxData[j] & shift) next_bit = 1;
	 			else next_bit = 0;
			
				for(i=0; i<8; i++)
				{
					// LOW BIT: 133.3 kHz
					if(next_bit == 0 && last_bit == 1)				// LOW bit after previous HIGH bit
					{
						while(g_count<2);
  						CCR2 = 15;
 						while(g_count<3);
 						P3OUT &= ~P3_nLH;							// P3.6 LOW		
 						CCR0 = 29;									// PWM Period: 30 clock cycles
					}
			
					// HIGH BIT: 121.2 kHz
					else if(next_bit == 1 && last_bit == 0)			// HIGH bit after previous LOW bit
					{
						while(g_count<2);
  						CCR2 = 17;
  						while(g_count<3);
  						P3OUT |= P3_nLH;							// P3.6 HIGH
						CCR0 = 32;									// PWM Period: 33 clock cycles
					}
			
					// Prepare next byte
 					shift <<= 1;
 					last_bit = next_bit;
 				
 					if(ucLFTxData[j] & shift)	next_bit = 1;
 					else next_bit = 0;
				
					// Wait while transmitting
 					while(g_count < 16);
 					g_count = 0;
				}
			}
	
			// Last 3 bits of data transmission
			while(g_count < 3);
	
			// Send final LOW bit: End of transmission
 			CCR2 = 15;
 			P3OUT &= ~P3_nLH;										// P3.6 LOW
 			CCR0 = 29;
 			g_count = 0;
 			while(g_count < 16);
			CCR0 = 0;
	
 			// Disable selected Driver
 			switch(Driver)
			{
				case Driver1:	P2OUT &= ~P2_EN1;	break;			// Driver 1 LOW
				case Driver2:	P2OUT &= ~P2_EN2;	break;			// Driver 2 LOW
				case Driver3:	P1OUT &= ~P1_EN3;	break;			// Driver 3 LOW
			}
		}
	}
}
