//******************************************************************************
//
//	SPI functions
//
//******************************************************************************

#include  <msp430x12x2.h>
#include  <defines.h>

// Initialize SPI interface
void MSP430_SPI_init(void)
{
  UCTL0 |= CHAR + SYNC + MM + SWRST; ;        	// 8-bit SPI Master
  U0TCTL |= SSEL0 + STC; 
  UBR00 = 0x28;			                     	// 4Mhz/40 = 100kHz
  UBR10 = 0x00;  		                 	 	// 40 = 0x[UBR10,UBR00]
  UCTL0 &= ~SWRST;                          	// Initialise USART state machine
}

// transmit telegram on SPI interface
void transmit_SPI(unsigned char ucLEN, unsigned char ucData[10])
{
	int i = 0;
	P1IE |= P1_BUSY;							// Enable interrupt: P1.0 BUSY
	
	// Transmit LEN
	while ((IFG2 & UTXIFG0) == 0);
	TXBUF0 = ucLEN;								// Send LEN
	while (!(U0TCTL & TXEPT));					// while transmission active

	// Transmit Data
	while(i < ucLEN)
	{
		interrupt_vector.field.busy = 0;
		g_count = 0;
		CCR0 = 4000;
		while(interrupt_vector.field.busy == 0 && g_count < 5);	// wait for BUSY rising edge or Timeout after 5 ms
		CCR0 = 0;

		while ((IFG2 & UTXIFG0) == 0);
		TXBUF0 = ucData[i];						// Send CMD
		while (!(U0TCTL & TXEPT));				// while transmission active
	
		i++;
	}
	
	g_count = 0;
	CCR0 = 4000;
	while(interrupt_vector.field.busy == 1 && g_count < 5);	// wait for BUSY falling edge or Timeout after 5 ms
	CCR0 = 0;
	P1IE &= ~P1_BUSY;							// Disable interrupt: P1.0 BUSY
}

// Receive telegram on SPI interface
void receive_SPI(unsigned char ucLEN)
{
	int i = 0;
	P1IE |= P1_BUSY;							// Enable interrupt: P1.0 BUSY
	do
	{
	  	while (!(U0TCTL & TXEPT));
	    TXBUF0 = 0x00;
	   	g_count = 0;
		CCR0 = 4000;
		while(interrupt_vector.field.busy == 1 && g_count < 5);	// wait for BUSY falling edge or Timeout after 5 ms
		CCR0 = 0;	
		ucSPIRxData[i] = RXBUF0;
		g_count = 0;
		CCR0 = 4000;
   	  	while(interrupt_vector.field.busy == 0 && g_count < 5);	// wait for BUSY rising edge or Timeout after 5 ms
   	  	CCR0 = 0;
   	  	i++;
	}while(i<ucLEN-1);
	
    TXBUF0 = 0x00;
    while (!(U0TCTL & TXEPT));
    g_count = 0;
	CCR0 = 4000;
   	while(interrupt_vector.field.busy == 1 && g_count < 5);	// wait for falling edge or Timeout after 5 ms
   	ucSPIRxData[i] = RXBUF0;
   	CCR0 = 0;
   	P1IE &= ~P1_BUSY;						// Disable interrupt: P1.0 BUSY
}
