//******************************************************************************
// Antenna board rev 8.0 #1
// 			DUAL ANTENNA LF-LF TRANSPONDER
//
//******************************************************************************
#include  <msp430x12x2.h>
#include  <defines.h>


// global variables
int g_count;
unsigned char   ucSPIRxData[10];
unsigned char   ucSPITxData[10];
unsigned char 	ucLFRxData[5]; 
unsigned char 	ucLFTxData[10];
union flag interrupt_vector;

void PORT_init(void)
{
	P3DIR |= P3_nLH;							// P3.6 output
  	P3DIR &= ~0x01;								// P3.0 input	
  	P3DIR |= 0x80;								// P3.7 output	
  	P3SEL |= 0x01;								// P3.0 AE5
  	
  	P1DIR &= ~P1_PB1;							// P1.5 input
  	P1DIR &= ~P1_PB2;							// P1.4 input
  	P1DIR |= 0x4E;                        		// P1.1, P1.2, P1.3, P1.6 output
  	P1SEL =  0x00;                        		// P1.3 not TA2 options
  	
  	P2DIR |= 0x18;								// P2.3, P2.4 output
  	
 	P1IES = 0x30;								// Interrupt edge select: 1.4, 1.5 high-to-low transition
  	P1IE =  0x30;								// Enable interrupt: P1.4, P1.5
  	P1IFG = 0x00;								// clear interrupt 
  	
  	P2IES = 0x00;								// Interrupt edge select: P2.1 - P2.2 low-to-high transition
  	P2IE =  0x00;
  	P2IFG = 0x00;
  	
  	P1OUT &= ~P1_EN3;							// P1.2 LOW
  	P2OUT &= ~P2_EN1; 							// P2.3 LOW
  	P2OUT &= ~P2_EN2;							// P2.4 LOW
}

PORT_reset(void)
{
	P1OUT &= ~P1_EN3;							// P1.2 LOW
  	P2OUT &= ~P2_EN1; 							// P2.3 LOW
  	P2OUT &= ~P2_EN2;							// P2.4 LOW
	
	P1IES |= 0x30;								// Set interrupt on high-low transition: S1, S2
	P1SEL &= ~P1_FTX;							// PWM off
	P1OUT &= ~P1_FTX;							// FTX LOW
	P3OUT &= ~P3_nLH;							// nLH LOW
}

void Waitms(int i)
{
	CCR0 = 4000;
	g_count = 0;
	while(g_count<i);
	CCR0 = 0;	
}

void Wait100us(int i)
{
	CCR0 = 400;
	g_count = 0;
	while(g_count<i);
	CCR0 = 0;	
}

void pushbuttons()
{
	// initialize Driver 1
	P1SEL |= P1_FTX; 							// 1.3 TA2 options
	P2OUT |= P2_EN1;							// Driver 1 HIGH
	
	while(!(P1IN & P1_PB1) || !(P1IN & P1_PB2))
	{
		if(!(P1IN & P1_PB1) && (P1IN & P1_PB2))	// HIGH BIT: 121.2 kHz
		{
			CCR2 = 17;
			CCR0 = 32;							// PWM Period: 33 clock cycles
			P3OUT |= P3_nLH;					// 121.2 kHz: '1'
  			
			while(!(P1IN & P1_PB1) && (P1IN & P1_PB2));
		}

		if((P1IN & P1_PB1) && !(P1IN & P1_PB2))	// LOW BIT: 133.3 kHz
		{
 			CCR2 = 15;
			CCR0 = 29;							// PWM Period: 30 clock cycles
			P3OUT &= ~P3_nLH;					// 133.3 kHz: '0'
		
			while((P1IN & P1_PB1) && !(P1IN & P1_PB2));
		}
		g_count = 0;
		
			
 		CCR2 = 15;
  		CCR0 = 29;	
		g_count = 0;
		while(!(P1IN & P1_PB1) && !(P1IN & P1_PB2))
		{
			while(g_count<2);
  			CCR2 = 17;
  			while(g_count<3);
  			P3OUT |= P3_nLH;					// 121.2 kHz: nLH = '1'
			CCR0 = 32;							// PWM Period: 33 clock cycles
			while(g_count < 16);
			g_count = 0;
			
			while(g_count<2);
 			CCR2 = 15;
 			while(g_count<3);
  			P3OUT &= ~P3_nLH;					// 133.3 kHz: nLH = '0'	
 			CCR0 = 29;							// PWM Period: 30 clock cycles		
   			while(g_count < 16);
   			g_count = 0;
		}
	}

	P2OUT &= ~P2_EN1;							// Driver 1 LOW
	CCR0 = 0;									// Timer1A off
}

void main(void)
{
	int i;
	
  	WDTCTL = WDTPW + WDTHOLD;               	// Stop WDT
  	
  	// initialize Pins
  	PORT_init();
  	
  	// initialize clock
  	BCSCTL1 = XTS + DIVA_0;						// ACLK on, 4 MHz
	
	do
  	{
  		IFG1 &= ~OFIFG;                    	 	// Clear OSCFault flag
  		for (i = 0xFF; i > 0; i--);         	// Time for flag to set
  	}
  	while ((IFG1 & OFIFG));                		// OSCFault flag still set?
	
    BCSCTL2 = SELM_3 + SELS;					// SMCLK
	
	// initialize PWM generator for FTX Pin
  	CCTL2 = OUTMOD_3;                   		// CCR2 set/reset
	TACTL = TASSEL_1 + MC_1;			   		// ACLK, upmode
	CCR0 = 0;									// Stop TimerA
	CCTL0 = CCIE;								// Enable CCR0 interrupt
	
	// initialize SPI
	MSP430_SPI_init();
	
	_BIS_SR(GIE);								// Global interrupt enable
	
	while(1)
	{
		if (P2IN & P2_WAKE)						// WAKE is active
		{
		 	WDEEN_OFF;
		 	CCR0 = 4000;
			g_count = 0;
			while(g_count < 100);				// Wait 100 ms
		}
		
		CCR0 = 0;
		P2IE = P2_WAKE;							// Enable interrupt for Wake
		P2IFG = 0x00;							// Clear interrupt
		
		interrupt_vector.field.S1 = 0;
		interrupt_vector.field.S2 = 0;
		interrupt_vector.field.wake = 0;
		
		WDEEN_ON;
		
		// Enter Low power mode 4
		_BIS_SR(LPM4);							// If interrupt occurs, wake up from LPM4
		
		do
  		{
  			IFG1 &= ~OFIFG;                     // Clear OSCFault flag
  			for (i = 0xFF; i > 0; i--);         // Time for flag to set
  		}
  		while ((IFG1 & OFIFG));                 // OSCFault flag still set?
  		
		if(interrupt_vector.field.wake == 1)
		{
			P2IE &= ~P2_WAKE;					// Disable interrupt for Wake
			PEPS();
		}
		else
		{
			P1OUT |= P1_LED;
			pushbuttons();
			P1OUT &= ~P1_LED;
		}
		
		PORT_reset();
	}
}

// Timer A0 interrupt service routine: CCR0
#pragma vector=TIMERA0_VECTOR
__interrupt void Timer_A0 (void)
{
	g_count++;
}

// Port 1 interrupt service routine
#pragma vector=PORT1_VECTOR
__interrupt void PORT1 (void)
{
	if(P1IFG & P1_PB1)
	{
		if(P1IES & P1_PB1)						// falling edge detected
		{
			P1IES &= ~P1_PB1;					// Set interrupt on low-high transition: S1
			LPM4_EXIT;
			interrupt_vector.field.S1 = 1;
		}
		else									// rising edge detected
		{
			interrupt_vector.field.S1 = 0;
			P1IES |= P1_PB1;					// Set interrupt on high-low transition: S1	
		}
	}
	
	if(P1IFG & P1_PB2)
	{
		if(P1IES & P1_PB2)						// falling edge detected
		{
			P1IES &= ~P1_PB2;					// Set interrupt on low-high transition: S2
			LPM4_EXIT;
			interrupt_vector.field.S2 = 1;
		}
		else									// rising edge detected
		{
			interrupt_vector.field.S2 = 0;
			P1IES |= P1_PB2;					// Set interrupt on high-low transition: S2
		}
	}

	if(P1IFG & P1_BUSY	)
	{
		if(P1IES & P1_BUSY	)					// falling edge detected
		{
			interrupt_vector.field.busy = 0;
			P1IES &= ~P1_BUSY	;				// Set interrupt on low-high transition: S2
		}
		else									// rising edge detected
		{	
			interrupt_vector.field.busy = 1;
			P1IES |= P1_BUSY	;				// Set interrupt on high-low transition: S2
		}
	}
	P1IFG = 0x00;
}

// Port 2 interrupt service routine
#pragma vector=PORT2_VECTOR
__interrupt void PORT2 (void)
{
	if(P2IFG & P2_WAKE)
	{
		LPM4_EXIT;								// Exit LPM4
		interrupt_vector.field.wake = 1;		// WAKE P2.1
	}
	
	if(P2IES & P2_EOBA)							// falling edge detected
	{
		if(P2IFG & P2_EOBA)	interrupt_vector.field.eoba = 0;
	}
	else										// rising edge detected
	{
		if(P2IFG & P2_EOBA)	interrupt_vector.field.eoba = 1;
	}
	
	P2IFG = 0x00;
}

// ADC interrupt service routine
#pragma vector=ADC10_VECTOR
__interrupt void ADC10_ISR (void)
{
	_BIC_SR_IRQ(LPM4_bits);
}
